﻿#include "mspell/monster-power-table.h"
#include "monster-race/race-ability-flags.h"
#include "player-ability/player-ability-types.h"

/*!
 * @brief 青魔法テーブル
 * @details
 * level,  smana,  %fail,  manedam,  %manefail,  use_stat, name
 */
const std::map<RF_ABILITY, const monster_power> monster_powers = {
    { RF_ABILITY::SHRIEK, { 1, 1, 10, 0, 15, A_CON, _("叫ぶ", "shriek") } },
    { RF_ABILITY::XXX1, { 10, 4, 35, 89, 40, A_INT, _("何か", "something") } },
    { RF_ABILITY::DISPEL, { 40, 35, 85, 0, 40, A_INT, _("魔力消去", "dispel-magic") } },
    { RF_ABILITY::ROCKET, { 35, 30, 80, 800, 70, A_STR, _("ロケット", "rocket") } },
    { RF_ABILITY::SHOOT, { 5, 1, 20, 18, 15, A_DEX, _("射撃", "arrow") } },
    { RF_ABILITY::XXX2, { 10, 4, 35, 89, 40, A_INT, _("何か", "arrows") } },
    { RF_ABILITY::XXX3, { 10, 4, 35, 89, 40, A_INT, _("何か", "missile") } },
    { RF_ABILITY::XXX4, { 10, 4, 35, 89, 40, A_INT, _("何か", "missiles") } },
    { RF_ABILITY::BR_ACID, { 20, 15, 55, 1600, 95, A_CON, _("酸のブレス", "breathe acid") } },
    { RF_ABILITY::BR_ELEC, { 20, 15, 55, 1600, 95, A_CON, _("電撃のブレス", "breathe lightning") } },
    { RF_ABILITY::BR_FIRE, { 20, 15, 55, 1600, 95, A_CON, _("炎のブレス", "breathe fire") } },
    { RF_ABILITY::BR_COLD, { 20, 15, 55, 1600, 95, A_CON, _("冷気のブレス", "breathe cold") } },
    { RF_ABILITY::BR_POIS, { 20, 15, 55, 800, 95, A_CON, _("毒のブレス", "breathe poison") } },
    { RF_ABILITY::BR_NETH, { 20, 15, 70, 550, 95, A_CON, _("地獄のブレス", "breathe nether") } },
    { RF_ABILITY::BR_LITE, { 20, 16, 70, 400, 95, A_CON, _("閃光のブレス", "breathe light") } },
    { RF_ABILITY::BR_DARK, { 20, 16, 70, 400, 95, A_CON, _("暗黒のブレス", "breathe dark") } },
    { RF_ABILITY::BR_CONF, { 20, 20, 70, 450, 95, A_CON, _("混乱のブレス", "breathe confusion") } },
    { RF_ABILITY::BR_SOUN, { 20, 20, 70, 450, 95, A_CON, _("轟音のブレス", "breathe sound") } },
    { RF_ABILITY::BR_CHAO, { 20, 20, 70, 600, 95, A_CON, _("カオスのブレス", "breathe chaos") } },
    { RF_ABILITY::BR_DISE, { 20, 16, 70, 500, 95, A_CON, _("劣化のブレス", "breathe disenchantment") } },
    { RF_ABILITY::BR_NEXU, { 30, 25, 80, 250, 95, A_CON, _("因果混乱のブレス", "breathe nexus") } },
    { RF_ABILITY::BR_TIME, { 35, 18, 80, 150, 95, A_CON, _("時間逆転のブレス", "breathe time") } },
    { RF_ABILITY::BR_INER, { 30, 25, 80, 200, 95, A_CON, _("遅鈍のブレス", "breathe inertia") } },
    { RF_ABILITY::BR_GRAV, { 30, 28, 90, 200, 95, A_CON, _("重力のブレス", "breathe gravity") } },
    { RF_ABILITY::BR_SHAR, { 20, 15, 70, 500, 95, A_CON, _("破片のブレス", "breathe shards") } },
    { RF_ABILITY::BR_PLAS, { 35, 15, 80, 150, 95, A_CON, _("プラズマのブレス", "breathe plasma") } },
    { RF_ABILITY::BR_FORC, { 30, 18, 70, 200, 95, A_CON, _("フォースのブレス", "breathe force") } },
    { RF_ABILITY::BR_MANA, { 30, 28, 80, 250, 95, A_CON, _("魔力のブレス", "breathe mana") } },
    { RF_ABILITY::BA_NUKE, { 25, 20, 95, 320, 80, A_INT, _("放射能球", "nuke ball") } },
    { RF_ABILITY::BR_NUKE, { 25, 15, 70, 800, 95, A_CON, _("放射性廃棄物のブレス", "breathe nuke") } },
    { RF_ABILITY::BA_CHAO, { 30, 32, 85, 400, 80, A_INT, _("純ログルス", "raw Logrus") } },
    { RF_ABILITY::BR_DISI, { 35, 40, 95, 150, 95, A_CON, _("分解のブレス", "breathe disintegration") } },

    { RF_ABILITY::BA_ACID, { 18, 13, 55, 630, 80, A_INT, _("アシッド・ボール", "acid ball") } },
    { RF_ABILITY::BA_ELEC, { 14, 10, 45, 316, 60, A_INT, _("サンダー・ボール", "lightning ball") } },
    { RF_ABILITY::BA_FIRE, { 20, 14, 60, 720, 80, A_INT, _("ファイア・ボール", "fire ball") } },
    { RF_ABILITY::BA_COLD, { 15, 11, 50, 320, 60, A_INT, _("アイス・ボール", "frost ball") } },
    { RF_ABILITY::BA_POIS, { 5, 3, 40, 48, 20, A_INT, _("悪臭雲", "stinking cloud") } },
    { RF_ABILITY::BA_NETH, { 25, 18, 70, 350, 80, A_INT, _("地獄球", "nether ball") } },
    { RF_ABILITY::BA_WATE, { 30, 22, 75, 350, 80, A_INT, _("ウォーター・ボール", "water ball") } },
    { RF_ABILITY::BA_MANA, { 44, 45, 85, 550, 95, A_INT, _("魔力の嵐", "mana storm") } },
    { RF_ABILITY::BA_DARK, { 40, 42, 90, 550, 95, A_INT, _("暗黒の嵐", "darkness storm") } },
    { RF_ABILITY::DRAIN_MANA, { 10, 5, 50, 0, 25, A_INT, _("魔力吸収", "drain mana") } },
    { RF_ABILITY::MIND_BLAST, { 25, 10, 60, 0, 30, A_INT, _("精神攻撃", "mind blast") } },
    { RF_ABILITY::BRAIN_SMASH, { 30, 14, 65, 0, 30, A_INT, _("脳攻撃", "brain smash") } },
    { RF_ABILITY::CAUSE_1, { 3, 1, 25, 24, 20, A_INT, _("軽傷", "cause light wounds") } },
    { RF_ABILITY::CAUSE_2, { 12, 2, 35, 64, 25, A_INT, _("重傷", "cause serious wounds") } },
    { RF_ABILITY::CAUSE_3, { 22, 6, 50, 150, 30, A_INT, _("致命傷", "cause critical wounds") } },
    { RF_ABILITY::CAUSE_4, { 32, 10, 70, 225, 35, A_INT, _("秘孔を突く", "cause mortal wounds") } },
    { RF_ABILITY::BO_ACID, { 13, 7, 40, 178, 40, A_INT, _("アシッド・ボルト", "acid bolt") } },
    { RF_ABILITY::BO_ELEC, { 10, 5, 35, 130, 35, A_INT, _("サンダー・ボルト", "lightning bolt") } },
    { RF_ABILITY::BO_FIRE, { 15, 9, 50, 210, 45, A_INT, _("ファイア・ボルト", "fire bolt") } },
    { RF_ABILITY::BO_COLD, { 12, 6, 35, 162, 40, A_INT, _("アイス・ボルト", "frost bolt") } },
    { RF_ABILITY::BA_LITE, { 40, 42, 90, 550, 95, A_INT, _("スター・バースト", "starburst") } },
    { RF_ABILITY::BO_NETH, { 25, 17, 60, 255, 60, A_INT, _("地獄の矢", "nether bolt") } },
    { RF_ABILITY::BO_WATE, { 25, 20, 65, 250, 60, A_INT, _("ウォーター・ボルト", "water bolt") } },
    { RF_ABILITY::BO_MANA, { 25, 24, 90, 400, 80, A_INT, _("魔力の矢", "mana bolt") } },
    { RF_ABILITY::BO_PLAS, { 25, 20, 80, 216, 60, A_INT, _("プラズマ・ボルト", "plasma bolt") } },
    { RF_ABILITY::BO_ICEE, { 25, 16, 60, 186, 60, A_INT, _("極寒の矢", "ice bolt") } },
    { RF_ABILITY::MISSILE, { 3, 1, 25, 12, 20, A_INT, _("マジック・ミサイル", "magic missile") } },
    { RF_ABILITY::SCARE, { 5, 3, 35, 0, 20, A_INT, _("恐慌", "scare") } },
    { RF_ABILITY::BLIND, { 10, 5, 40, 0, 20, A_INT, _("盲目", "blind") } },
    { RF_ABILITY::CONF, { 10, 5, 40, 0, 20, A_INT, _("パニック・モンスター", "confuse") } },
    { RF_ABILITY::SLOW, { 10, 5, 40, 0, 20, A_INT, _("スロウ・モンスター", "slow") } },
    { RF_ABILITY::HOLD, { 10, 5, 40, 0, 20, A_INT, _("スリープ・モンスター", "sleep") } },

    { RF_ABILITY::HASTE, { 20, 10, 70, 0, 40, A_INT, _("スピード", "speed") } },
    { RF_ABILITY::HAND_DOOM, { 45, 120, 95, 0, 60, A_INT, _("破滅の手", "the Hand of Doom") } },
    { RF_ABILITY::HEAL, { 20, 15, 70, 0, 20, A_WIS, _("体力回復", "heal-self") } },
    { RF_ABILITY::INVULNER, { 45, 65, 80, 0, 60, A_INT, _("無傷の球", "make invulnerable") } },
    { RF_ABILITY::BLINK, { 5, 1, 30, 0, 20, A_INT, _("ショート・テレポート", "blink-self") } },
    { RF_ABILITY::TPORT, { 15, 8, 40, 0, 30, A_INT, _("テレポート", "teleport-self") } },
    { RF_ABILITY::WORLD, { 40, 999, 99, 0, 80, A_INT, _("ザ・ワールド", "The world") } },
    { RF_ABILITY::SPECIAL, { 1, 0, 0, 0, 15, A_INT, _("何か", "something") } },
    { RF_ABILITY::TELE_TO, { 15, 8, 50, 0, 30, A_INT, _("引きよせる", "teleport to") } },
    { RF_ABILITY::TELE_AWAY, { 20, 13, 80, 0, 30, A_INT, _("テレポート・アウェイ", "teleport away") } },
    { RF_ABILITY::TELE_LEVEL, { 30, 40, 95, 0, 40, A_INT, _("テレポート・レベル", "teleport level") } },
    { RF_ABILITY::PSY_SPEAR, { 35, 30, 80, 350, 70, A_INT, _("光の剣", "psycho-spear") } },
    { RF_ABILITY::DARKNESS, { 5, 1, 20, 0, 15, A_INT, _("暗闇", "create darkness") } },
    { RF_ABILITY::TRAPS, { 5, 1, 20, 0, 15, A_DEX, _("トラップ創造", "create traps") } },
    { RF_ABILITY::FORGET, { 15, 3, 40, 0, 30, A_INT, _("記憶喪失", "cause amnesia") } },
    { RF_ABILITY::RAISE_DEAD, { 30, 30, 70, 0, 40, A_INT, _("死者復活", "raise dead") } },
    { RF_ABILITY::S_KIN, { 40, 70, 85, 0, 45, A_INT, _("援軍を呼ぶ", "summon aid") } },
    { RF_ABILITY::S_CYBER, { 45, 90, 90, 0, 50, A_INT, _("サイバーデーモンの召喚", "summon Cyberdemons") } },
    { RF_ABILITY::S_MONSTER, { 25, 20, 65, 0, 30, A_INT, _("モンスターの召喚", "summon a monster") } },
    { RF_ABILITY::S_MONSTERS, { 35, 30, 75, 0, 40, A_INT, _("複数のモンスターの召喚", "summon monsters") } },
    { RF_ABILITY::S_ANT, { 25, 25, 65, 0, 25, A_INT, _("アリの召喚", "summon ants") } },
    { RF_ABILITY::S_SPIDER, { 25, 20, 60, 0, 25, A_INT, _("蜘蛛の召喚", "summon spiders") } },
    { RF_ABILITY::S_HOUND, { 35, 26, 75, 0, 40, A_INT, _("ハウンドの召喚", "summon hounds") } },
    { RF_ABILITY::S_HYDRA, { 30, 23, 70, 0, 35, A_INT, _("ヒドラの召喚", "summon hydras") } },
    { RF_ABILITY::S_ANGEL, { 40, 50, 85, 0, 40, A_INT, _("天使の召喚", "summon an angel") } },
    { RF_ABILITY::S_DEMON, { 35, 50, 80, 0, 35, A_INT, _("デーモンの召喚", "summon a daemon") } },
    { RF_ABILITY::S_UNDEAD, { 30, 30, 75, 0, 35, A_INT, _("アンデッドの召喚", "summon an undead") } },
    { RF_ABILITY::S_DRAGON, { 39, 70, 80, 0, 40, A_INT, _("ドラゴンの召喚", "summon a dragon") } },
    { RF_ABILITY::S_HI_UNDEAD, { 43, 85, 85, 0, 45, A_INT, _("上級アンデッドの召喚", "summon Greater Undead") } },
    { RF_ABILITY::S_HI_DRAGON, { 46, 90, 85, 0, 45, A_INT, _("古代ドラゴンの召喚", "summon Ancient Dragon") } },
    { RF_ABILITY::S_AMBERITES, { 48, 120, 90, 0, 50, A_INT, _("アンバーの王族の召喚", "summon Lords of Amber") } },
    { RF_ABILITY::S_UNIQUE, { 50, 150, 95, 0, 50, A_INT, _("ユニークモンスターの召喚", "summon Unique Monsters") } },
};

/*!
 * @brief モンスター魔法名テーブル
 */
const std::map<RF_ABILITY, concptr> monster_powers_short = {
    { RF_ABILITY::SHRIEK, _("叫ぶ", "Shriek") },
    { RF_ABILITY::XXX1, _("何か", "Something") },
    { RF_ABILITY::DISPEL, _("魔力消去", "Dispel-magic") },
    { RF_ABILITY::ROCKET, _("ロケット", "Rocket") },
    { RF_ABILITY::SHOOT, _("射撃", "Arrow") },
    { RF_ABILITY::XXX2, _("何か", "Arrows") },
    { RF_ABILITY::XXX3, _("何か", "Missile") },
    { RF_ABILITY::XXX4, _("何か", "Missiles") },
    { RF_ABILITY::BR_ACID, _("酸", "Acid") },
    { RF_ABILITY::BR_ELEC, _("電撃", "Lightning") },
    { RF_ABILITY::BR_FIRE, _("火炎", "Fire") },
    { RF_ABILITY::BR_COLD, _("冷気", "Cold") },
    { RF_ABILITY::BR_POIS, _("毒", "Poison") },
    { RF_ABILITY::BR_NETH, _("地獄", "Nether") },
    { RF_ABILITY::BR_LITE, _("閃光", "Light") },
    { RF_ABILITY::BR_DARK, _("暗黒", "Dark") },
    { RF_ABILITY::BR_CONF, _("混乱", "Confusion") },
    { RF_ABILITY::BR_SOUN, _("轟音", "Sound") },
    { RF_ABILITY::BR_CHAO, _("カオス", "Chaos") },
    { RF_ABILITY::BR_DISE, _("劣化", "Disenchantment") },
    { RF_ABILITY::BR_NEXU, _("因果混乱", "Nexus") },
    { RF_ABILITY::BR_TIME, _("時間逆転", "Time") },
    { RF_ABILITY::BR_INER, _("遅鈍", "Inertia") },
    { RF_ABILITY::BR_GRAV, _("重力", "Gravity") },
    { RF_ABILITY::BR_SHAR, _("破片", "Shards") },
    { RF_ABILITY::BR_PLAS, _("プラズマ", "Plasma") },
    { RF_ABILITY::BR_FORC, _("フォース", "Force") },
    { RF_ABILITY::BR_MANA, _("魔力", "Mana") },
    { RF_ABILITY::BA_NUKE, _("放射能球", "Nuke") },
    { RF_ABILITY::BR_NUKE, _("放射性廃棄物", "Nuke") },
    { RF_ABILITY::BA_CHAO, _("純ログルス", "Logrus") },
    { RF_ABILITY::BR_DISI, _("分解", "Disintegration") },

    { RF_ABILITY::BA_ACID, _("酸", "Acid") },
    { RF_ABILITY::BA_ELEC, _("電撃", "Lightning") },
    { RF_ABILITY::BA_FIRE, _("火炎", "Fire") },
    { RF_ABILITY::BA_COLD, _("冷気", "Frost") },
    { RF_ABILITY::BA_POIS, _("悪臭雲", "Stinking Cloud") },
    { RF_ABILITY::BA_NETH, _("地獄球", "Nether") },
    { RF_ABILITY::BA_WATE, _("ウォーター", "Water") },
    { RF_ABILITY::BA_MANA, _("魔力の嵐", "Mana storm") },
    { RF_ABILITY::BA_DARK, _("暗黒の嵐", "Darkness storm") },
    { RF_ABILITY::DRAIN_MANA, _("魔力吸収", "Drain mana") },
    { RF_ABILITY::MIND_BLAST, _("精神攻撃", "Mind blast") },
    { RF_ABILITY::BRAIN_SMASH, _("脳攻撃", "Brain smash") },
    { RF_ABILITY::CAUSE_1, _("軽傷", "Cause Light Wound") },
    { RF_ABILITY::CAUSE_2, _("重傷", "Cause Serious Wound") },
    { RF_ABILITY::CAUSE_3, _("致命傷", "Cause Critical Wound") },
    { RF_ABILITY::CAUSE_4, _("秘孔を突く", "Cause Mortal Wound") },
    { RF_ABILITY::BO_ACID, _("酸", "Acid") },
    { RF_ABILITY::BO_ELEC, _("電撃", "Lightning") },
    { RF_ABILITY::BO_FIRE, _("火炎", "Fire") },
    { RF_ABILITY::BO_COLD, _("冷気", "Frost") },
    { RF_ABILITY::BA_LITE, _("スターバースト", "Starburst") },
    { RF_ABILITY::BO_NETH, _("地獄の矢", "Nether") },
    { RF_ABILITY::BO_WATE, _("ウォーター", "Water") },
    { RF_ABILITY::BO_MANA, _("魔力の矢", "Mana") },
    { RF_ABILITY::BO_PLAS, _("プラズマ", "Plasma") },
    { RF_ABILITY::BO_ICEE, _("極寒", "Ice") },
    { RF_ABILITY::MISSILE, _("マジックミサイル", "Magic missile") },
    { RF_ABILITY::SCARE, _("恐慌", "Scare") },
    { RF_ABILITY::BLIND, _("盲目", "Blind") },
    { RF_ABILITY::CONF, _("混乱", "Confuse") },
    { RF_ABILITY::SLOW, _("減速", "Slow") },
    { RF_ABILITY::HOLD, _("睡眠", "Sleep") },

    { RF_ABILITY::HASTE, _("加速", "Speed") },
    { RF_ABILITY::HAND_DOOM, _("破滅の手", "Hand of doom") },
    { RF_ABILITY::HEAL, _("体力回復", "Heal-self") },
    { RF_ABILITY::INVULNER, _("無傷の球", "Invulnerable") },
    { RF_ABILITY::BLINK, _("ショートテレポート", "Blink") },
    { RF_ABILITY::TPORT, _("テレポート", "Teleport") },
    { RF_ABILITY::WORLD, _("時を止める", "The world") },
    { RF_ABILITY::SPECIAL, _("何か", "Something") },
    { RF_ABILITY::TELE_TO, _("引きよせる", "Teleport to") },
    { RF_ABILITY::TELE_AWAY, _("テレポートアウェイ", "Teleport away") },
    { RF_ABILITY::TELE_LEVEL, _("テレポートレベル", "Teleport level") },
    { RF_ABILITY::PSY_SPEAR, _("光の剣", "Psycho-spear") },
    { RF_ABILITY::DARKNESS, _("暗闇", "Create darkness") },
    { RF_ABILITY::TRAPS, _("トラップ創造", "Create traps") },
    { RF_ABILITY::FORGET, _("記憶喪失", "Amnesia") },
    { RF_ABILITY::RAISE_DEAD, _("死者復活", "Raise dead") },
    { RF_ABILITY::S_KIN, _("援軍", "Aid") },
    { RF_ABILITY::S_CYBER, _("サイバーデーモン", "Cyberdeamons") },
    { RF_ABILITY::S_MONSTER, _("モンスター", "A monster") },
    { RF_ABILITY::S_MONSTERS, _("複数のモンスター", "Monsters") },
    { RF_ABILITY::S_ANT, _("蟻", "Ants") },
    { RF_ABILITY::S_SPIDER, _("蜘蛛", "Spiders") },
    { RF_ABILITY::S_HOUND, _("ハウンド", "Hounds") },
    { RF_ABILITY::S_HYDRA, _("ヒドラ", "Hydras") },
    { RF_ABILITY::S_ANGEL, _("天使", "Angel") },
    { RF_ABILITY::S_DEMON, _("悪魔", "Daemon") },
    { RF_ABILITY::S_UNDEAD, _("アンデッド", "Undead") },
    { RF_ABILITY::S_DRAGON, _("ドラゴン", "Dragon") },
    { RF_ABILITY::S_HI_UNDEAD, _("上級アンデッド", "Greater Undead") },
    { RF_ABILITY::S_HI_DRAGON, _("古代ドラゴン", "Ancient Dragon") },
    { RF_ABILITY::S_AMBERITES, _("アンバーの王族", "Lords of Amber") },
    { RF_ABILITY::S_UNIQUE, _("ユニーク", "Unique monsters") },
};
